<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * CUL Activity Stream Block
 *
 * @package    block
 * @subpackage culactivity_stream
 * @copyright  2013 Amanda Doughty <amanda.doughty.1@city.ac.uk>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 *
 */

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot.'/blocks/culactivity_stream/locallib.php');


/**
 * Main culactivity_stream block class for display on course pages
 */
class block_culactivity_stream extends block_base {

    /*
     * The htmltitle of the block to be displayed in the block title area.
     * @public string $htmltitle
     */

    /**
     * Initialiser
     */
    public function init() {
        global $COURSE;
        if ($COURSE->id != SITEID) {
            $this->title = get_string('activitystreamcourse', 'block_culactivity_stream');
        } else {
            $this->title = get_string('activitystreamsite', 'block_culactivity_stream');
        }
    }

    /**
     * Fetches content for the block when displayed
     * @return string Block content
     */
    public function get_content() {
        global $COURSE;

        if ($this->content !== null) {
            return $this->content;
        }

        if (empty($this->instance)) {
            $this->content = '';
            return $this->content;
        }

        $limitnum = 5;
        $page = optional_param('block_culactivity_stream_page', 1, PARAM_RAW);
        $limitfrom = $page > 1 ? ($page * $limitnum) - $limitnum : 0;

        $this->content = new stdClass;
        $this->content->text = '';
        $this->content->footer = '';
        list($count, $notifications) = block_culactivity_stream_get_notifications($COURSE->id, 0, $limitfrom, $limitnum);
        $renderer = $this->page->get_renderer('block_culactivity_stream');
        $this->content->text = $renderer->culactivity_stream_reload();
        $this->content->text .= $renderer->culactivity_stream($notifications, $page, $this->instance->id);

        $prev = false;
        $next = false;

        if ($page > 1) {
            // Add a 'newer' link.
            $prev = $page - 1;
        }

        if (($limitfrom + $limitnum) < $count) {
            // Add an 'older' link.
            $next = $page + 1;
        }

        $this->content->text .= $renderer->culactivity_stream_pagination($prev, $next);

        $this->page->requires->yui_module('moodle-block_culactivity_stream-scroll',
                'M.block_culactivity_stream.scroll.init',
                [[
                    'limitnum' => $limitnum,
                    'count' => $count,
                    'courseid' => $COURSE->id,
                    'returnurl' => $this->page->url->out(),
                    'instanceid' => $this->instance->id
                ]]
            );

        return $this->content;
    }

    /**
     * Returns a list of formats, and whether the block
     * should be displayed within them.
     * @return array(string => boolean) List of formats
     */
    public function applicable_formats() {
        return array('site' => true, 'my' => true, 'course' => true);
    }

    /**
     * Returns whether multiple culactivity_stream blocks
     * are allowed within the same course.
     * In the case of culactivity_stream, no.
     * @return boolean Multiple blocks?
     * @version 2010081801
     * @since 2010081801
     */
    public function instance_allow_multiple() {
        return false;
    }

    public function hide_header() {
        return false;
    }

    public function has_config() {
        return false;
    }

    /**
     * Return the plugin config settings for external functions.
     *
     * @return stdClass the configs for both the block instance and plugin
     * @since Moodle 3.8
     */
    public function get_config_for_external() {
        // Return all settings for all users since it is safe (no private keys, etc..).
        $configs = !empty($this->config) ? $this->config : new stdClass();

        return (object) [
            'instance' => $configs,
            'plugin' => new stdClass(),
        ];
    }
}

